package frequtils;

#
# frequency hash utilities module
#

use strict;
use warnings;
use Function::Parameters qw(:strict);

use Exporter 'import';
our @EXPORT = qw(build_freq_hash unique_values distinct_nonunique_values distinct_values);

#
# %freq = build_freq_hash( @array ):
#	Build a frequency hash of the elements of @array, i.e. a hash
#	mapping each element (key) to the frequency of that element in @array,
#
fun build_freq_hash( @array )
{
	my %freq; map { $freq{$_}++ } @array;	 # array element -> frequency
	return %freq;
}

#
# @uniq = unique_values( %freq ):
#	Deliver all non-repeated values from a %freq hash
#	in an undetermined order
#
fun unique_values( %freq )
{
	my @uniq = grep { $freq{$_} == 1 } keys %freq;
	return @uniq;
}

#
# @nonuniq = distinct_nonunique_values( %freq ):
#	Deliver all repeated (non-unique) values from %freq
#	in an undetermined order
#
fun distinct_nonunique_values( %freq )
{
	my %seen;				 # elements we've already seen
	my @nonuniq = grep 			 # distinct non-unique elements
		{ $freq{$_} > 1 && ! $seen{$_}++ } keys %freq;
	return @nonuniq;
}

#
# @distinct = distinct_values( %freq ):
#	Deliver all distinct values from %freq 
#	in an undetermined order
#
fun distinct_values( %freq )
{
	return keys %freq;
}


1;
